/* ============================================================================
 * Copyright (c) Vincent Hardy 2009. All rights reserved
 * ===========================================================================*/

(function () {

var svgNS = "http://www.w3.org/2000/svg";

// Quotes from http://en.wikiquote.org/wiki/American_proverbs
var quotes = [
    [["If you wait till the last minute,"],["it'll only take a minute"]],
    [["Just because you can,"], ["doesn't mean you should"]],
    [["Knowledge is like manure,"], ["it's only good when spread"]],
    [["Fool me once, shame on you."], ["Fool me twice, shame on me."],
     ["Fool me thrice, shame on both of us"]],
    [["Procrastination is the key"], ["to failure"]],
    [["Imitation is the sincerest"], ["form of flattery"]],
    [["I'd be a failure if it weren't for"], ["all the mistakes I've made"]],
    [["It's never too late to"],["be what you might have been."]],
    [["Listen and silent are spelled"], ["using the same letters"]],
    [["Never enter a mud fight with a pig"]],
    [["People who don't talk are"], ["thought to be stupid;"],
        ["people who do prove it"]],
    [["Talk is cheap"]],
    [["What comes around goes around"]],
    [["When life hands you lemons"], ["make lemonade"]]
];

var fontFamilies = [
    "GoodDogRegular"
    ];

function loadDemo(Y, audio) {
    var TextEffects = sw.TextEffects, effects = TextEffects.effects,
        effectName = Y.one('#effectName'), curQuote = 0, previousEffect,
        curFont = 0, firstEffect = {effect: null, name: ""};

    function getEndHandler(node, name) {
        return function () {
            node.get('parentNode').removeChild(node);
            if (effectName.get('textContent') === name) {
                effectName.set('textContent', '');
            }
        }
    }

    function getSetTitleHandler (title) {
        return function () {
            effectName.set('textContent', title);
        }
    }

    Y.each(effects, function (e, effectName) {
        if (effects.hasOwnProperty(effectName) === true) {
            var textParent = Y.one(document.createElementNS(svgNS, 'g')),
                textGroup = Y.one("#textGroup"), effect,
                quoteLines = quotes[curQuote], text, lines = [];

            for (var i = 0; i < quoteLines.length; i++) {
                text = Y.one(document.createElementNS(svgNS, 'text'));
                lines.push(text);

                text.addClass("text-effects");
                text.set("textContent", quoteLines[i]);
                text.set("y", (-((quoteLines.length - 1) / 2) + i) + 'em');
                textParent.set('font-family', "'" + fontFamilies[curFont] + "'");
                textGroup.appendChild(textParent);
                textParent.appendChild(text);
            }

            effect = e.applyEffect(lines, {});
            effect.setStartState();

            // We chain effects: one starts when the previous one ends.
            if (previousEffect !== undefined) {
                previousEffect.onEnd(effect, 3.1);
            } else {
                firstEffect.effect = effect;
                firstEffect.name = effectName;
            }

            // When the effect ends, we remove the text from the tree
            effect.onEnd(getEndHandler(textParent, effectName), 2.6);

            // When the effect begins, we set its name in the title area
            effect.onBegin(getSetTitleHandler(effectName));

            previousEffect = effect;

            curQuote++;
            if (curQuote >= quotes.length) {
                curQuote = 0;
            }
            curFont++;
            if (curFont >= fontFamilies.length) {
                curFont = 0;
            }
        }
    });

    var body = Y.one('body'),
        startHandler = body.on('click', function () {
            startHandler.detach();
            Y.one('#info').setStyle('display', 'none');
            Y.one('.demoTitle').setStyle('display', 'inline');
            firstEffect.effect.run(2);
            getSetTitleHandler(firstEffect.name)();
            if (audio !== undefined) {
                audio.play();
            }
       })
}

YUI({
    groups: {
        svg: {
            combine: false,
            base: "../res/3.0/scripts/yui/",
            modules:  {
                "node-svg": {
                    path: 'svg/node-svg-debug.js',
                    requires: ['node']
                },
                "dom-style-svg": {
                    path: 'svg/dom-style-svg-debug.js',
                    requires: ['node', 'dom-svg']
                },
                "dom-svg": {
                    path: 'svg/dom-svg-debug.js',
                    requires: ['dom']
                },
                "anim-timing": {
                    path: 'anim-timing/anim-timing-debug.js',
                    requires: ['anim', 'node-svg']
                },
                "anim-svg": {
                    path: 'svg/anim-svg-debug.js',
                    requires: ['dom', 'dom-svg', 'anim', 'dom-style-svg']
                },
                "text-effects": {
                    path: '../tools/text-effects.js',
                    requires: ['anim-svg', 'anim-timing']
                },
                "asset-loader": {
                    path: '../tools/asset-loader.js',
                    requires: ['node']
                }
            }
        }
    }
}).use('anim-svg', 'anim-timing', 'text-effects', 'asset-loader', function(Y) {
    Y.on('contentready', function () {
        // Let the user know we are loading audio now
        var info = Y.one('#info');
        info.set('textContent', 'loading audio');

        // Load the demo's assets.
        var loader = new sw.AssetLoader(), audio;

        audio = loader.addAudioAsset("../res/3.0/audio/text-music-only",
                                      ["mp3", "ogg", "wav"]);
        loader.on('all-loaded', function () {
            info.set('textContent', 'click to start');
            loadDemo(Y, audio);
        });
        loader.on('error', function () {
            info.setStyle('textContent', 'click to start');
            loadDemo(Y, undefined);
        });
        
        loader.loadQueuedAssets();
    }, 'body');
});

})();